# frozen_string_literal: true

require 'addressable'
require_relative 'reference'
require_relative 'referenced_object'

class DistributedPress
  module V1
    module Social
      # Fetches ActivityStreams from different instances by
      # instantiating clients.
      class Dereferencer
        # @return [DistributedPress::V1::Social::Client]
        attr_reader :client

        # We only need the client
        def _dump(_)
          Marshal.dump(client)
        end
 
        def self._load(client)
          new(client: Marshal.load(client))
        end

        # @param :client [DistributedPress::V1::Social::Client]
        def initialize(client:)
          @client = client
          @parser =
            proc do |body, format|
              next HTTParty::Parser.call(body, format || :plain) unless body&.starts_with? '{'

              ReferencedObject.new(object: HTTParty::Parser.call(body, :json), dereferencer: self)
            end
        end

        # Fetch a URI
        #
        # @param :uri [String, Addressable::URI]
        # @return [HTTParty::Response]
        def get(uri:)
          uri = uris(uri)

          clients(uri).get(endpoint: uri.path, parser: @parser)
        end

        # Gets a client for a URI
        #
        # @param :uri [Addressable::URI]
        # @return [DistributedPress::V1::Social::Client]
        def clients(uri)
          @@clients ||= {}
          @@clients[uri.origin] ||=
            client.class.new(
              url: uri.origin,
              public_key_url: client.public_key_url,
              private_key_pem: client.private_key.to_s,
              logger: client.logger,
              cache_store: client.class.cache_store
            )
        end

        # Gets a reference for a URI and indexes it by the complete
        # and normalized URI
        #
        # @param :uri [String, Addressable::URI]
        # @return [DistributedPress::V1::Social::Reference]
        def references(uri)
          @@references ||= {}
          @@references[uri.to_s] ||= Reference.new(uri: uri.to_s, dereferencer: self)
        end

        # Make sure we're getting a normalized Addressable::URI
        #
        # @param :uri [String, Addressable::URI]
        # @return [Addressable::URI]
        def uris(uri)
          @@uris ||= {}
          @@uris[uri.to_s] ||=
            (if uri.is_a? Addressable::URI
               uri
             else
               Addressable::URI.parse(uri)
             end).normalize
        end

      end
    end
  end
end
